@@ -2,13 +2,12 @@ package Date::Baha::i;
 
 # Package declarations {{{
 use strict;
-use vars qw($VERSION); $VERSION = '0.12';
-use base qw(Exporter);
+use vars '$VERSION'; $VERSION = '0.13';
+use base 'Exporter';
 use vars qw(@EXPORT @EXPORT_OK);
 @EXPORT = @EXPORT_OK = qw(
     as_string
     cycles
-    date
     days
     days_of_the_week
     from_bahai
@@ -23,8 +22,6 @@ use Date::Calc qw(
     Add_Delta_Days
     Date_to_Days
     Day_of_Week
-    Day_of_Year
-    Delta_Days
     Timezone
     leap_year
 );
@@ -38,7 +35,7 @@ use constant FEBRUARY => 2;
 use constant MARCH => 3;
 use constant SHARAF => 16;
 use constant LAST_START_DAY => 2;   # First day of the fast.
-use constant YEAR_START_DAY => 21;  # Spring equinox.
+use constant YEAR_START_DAY => 21;  # Vernal equinox.
 use constant LEAP_START_DAY => 26;  # The intercalary days.
 use constant FIRST_YEAR  => 1844;
 use constant ADJUST_YEAR => 1900;
@@ -89,9 +86,8 @@ use constant MONTH_DAY => qw(
 
 # NOTE: Trailing 0's are stripped, resulting in incorrect
 # computations if certain decimals are not quoted.
+#   Month name   => [Number, Start, End],  # Non-leap year day span
 use constant MONTHS => {
-# Month => [Month num, Date span], # Non-leap year day span
-
     "Baha"       => [ 0,  '3.21',  '4.08'],  # 80,  98
     "Jalal"      => [ 1,  '4.09',  '4.27'],  # 99, 117
     "Jamal"      => [ 2,  '4.28',  '5.16'],  #118, 136
@@ -123,23 +119,24 @@ use constant DOW_NAME => qw(
     Istijlal
     Istiqlal
 );
+
 use constant HOLY_DAYS => {
-    # Work suspended:
-    "Naw Ruz"                   => [ 3, 21],
-    "First Day of Ridvan"       => [ 4, 21],
-    "Ninth Day of Ridvan"       => [ 4, 29],
-    "Twelfth Day of Ridvan"     => [ 5,  2],
-    "Declaration of the Bab"    => [ 5, 23],
-    "Ascension of Baha'u'llah"  => [ 5, 29],
-    "Martyrdom of the Bab"      => [ 7,  9],
-    "Birth of the Bab"          => [10, 20],
-    "Birth of Baha'u'llah"      => [11, 12],
+    # Work suspended':
+    "Naw Ruz"                   => [ '3.21'],
+    "First Day of Ridvan"       => [ '4.21'],
+    "Ninth Day of Ridvan"       => [ '4.29'],
+    "Twelfth Day of Ridvan"     => [ '5.02'],
+    "Declaration of the Bab"    => [ '5.23'],
+    "Ascension of Baha'u'llah"  => [ '5.29'],
+    "Martyrdom of the Bab"      => [ '7.09'],
+    "Birth of the Bab"          => ['10.20'],
+    "Birth of Baha'u'llah"      => ['11.12'],
     # Work not suspended:
-    "Day of the Covenant"       => [11, 26],
-    "Ascension of 'Abdu'l-Baha" => [11, 28],
-    "Ayyam-i-Ha"                => [ 2, 26,  4],  # 5 days in leap years
-    "The Fast"                  => [ 3,  2, 19],
-    "Days of Ridvan"            => [ 4, 21, 12],
+    "Ayyam-i-Ha"                => [ '2.26',  4],  # 5 days in leap years
+    "The Fast"                  => [ '3.02', 19],
+    "Days of Ridvan"            => [ '4.21', 12],
+    "Day of the Covenant"       => ['11.26'],
+    "Ascension of 'Abdu'l-Baha" => ['11.28'],
 };
 # }}}
 
@@ -152,7 +149,6 @@ sub days_of_the_week { return DOW_NAME }
 sub holy_days { return HOLY_DAYS }
 # }}}
 
-sub date { goto &to_bahai }
 sub to_bahai {  # {{{
     my %args = @_;
     # readability++
@@ -197,9 +193,8 @@ sub from_bahai {  # {{{
     my %args = @_;
 
     # Figure out the year.
-    my $year = $args{month} > SHARAF || $args{month} == -1
-        ? $args{year} + FIRST_YEAR
-        : $args{year} + FIRST_YEAR - 1;
+    my $year = $args{year} + FIRST_YEAR;
+    $year-- unless $args{month} > SHARAF || $args{month} == -1;
 
     # Reset the month number if we are given Ayyam-i-Ha.
     $args{month} = 0 if $args{month} == -1;
@@ -228,6 +223,8 @@ sub as_string {  # {{{
 
     my $date;
 
+    my $is_ayyam_i_ha = $date_hash->{month} == -1 ? 1 : 0;
+
     if (!$args{size} && $args{numeric} && $args{alpha}) {
         # short alpha-numeric
         $date .= sprintf '%s (%d), %s (%d) of %s (%d), year %d, %s (%d) of %s (%d)',
@@ -238,10 +235,15 @@ sub as_string {  # {{{
     }
     elsif ($args{size} && $args{numeric} && $args{alpha}) {
         # long alpha-numeric
-        $date .= sprintf '%s week day %s, %s day %s of the %s month %s, year %s (%d), %s year %s of the %s vahid %s of the %s kull-i-shay',
+        # XXX Fugly hacking begins.
+        my $month_string = $is_ayyam_i_ha ? '%s%s' : 'the %s month %s';
+
+        $date .= sprintf
+            "%s week day %s, %s day %s of $month_string, year %s (%d), %s year %s of the %s vahid %s of the %s kull-i-shay",
             ordinate ($date_hash->{dow}), $date_hash->{dow_name},
             ordinate ($date_hash->{day}), $date_hash->{day_name},
-            ordinate ($date_hash->{month}), $date_hash->{month_name},
+            ($is_ayyam_i_ha ? '' : ordinate ($date_hash->{month})),
+            $date_hash->{month_name},
             lc (Lingua::Num2Word::cardinal ('en', $date_hash->{year})),
             $date_hash->{year},
             ordinate ($date_hash->{cycle_year}),
@@ -252,30 +254,30 @@ sub as_string {  # {{{
     }
     elsif (!$args{size} && $args{numeric}) {
         # short numeric
-        $date .= sprintf '%s, %s/%s/%s',
-            @$date_hash{qw(dow month day year)};
+        $date .= sprintf '%s/%s/%s', @$date_hash{qw(month day year)};
     }
     elsif ($args{size} && $args{numeric}) {
         # long numeric
-        $date .= sprintf '%s day of the week, %s day of the %s month, year %s, %s year of the %s vahid of the %s kull-i-shay',
-            ordinate ($date_hash->{dow}),
-            ordinate ($date_hash->{day}),
-            ordinate ($date_hash->{month}),
-            $date_hash->{year},
-            ordinate ($date_hash->{cycle_year}),
-            ordinate ($date_hash->{cycle}),
+        $date .= sprintf
+            '%s day of the week, %s day of the %s month, year %s, %s year of the %s vahid of the %s kull-i-shay',
+            ordinate ($date_hash->{dow}), ordinate ($date_hash->{day}),
+            ordinate ($date_hash->{month}), $date_hash->{year},
+            ordinate ($date_hash->{cycle_year}), ordinate ($date_hash->{cycle}),
             ordinate ($date_hash->{kull_i_shay});
     }
     elsif (!$args{size} && $args{alpha}) {
         # short alpha
         $date .= sprintf '%s, %s of %s, %s of %s',
             @$date_hash{qw(
-                dow_name month_name day_name year_name cycle_name
+                dow_name day_name month_name year_name cycle_name
             )};
     }
     else {
         # long alpha
-        $date .= sprintf 'week day %s, day %s of month %s, year %s of year %s of the vahid %s of the %s kull-i-shay',
+        my $month_string = $is_ayyam_i_ha ? '%s' : 'month %s';
+
+        $date .= sprintf
+            "week day %s, day %s of $month_string, year %s of year %s of the vahid %s of the %s kull-i-shay",
             @$date_hash{qw(dow_name day_name month_name)},
             lc (Lingua::Num2Word::cardinal ('en', $date_hash->{year})),
             @$date_hash{qw(year_name cycle_name)},
@@ -312,13 +314,12 @@ sub as_string {  # {{{
 sub next_holy_day {  # {{{
     my ($year, $month, $day) = @_;
 
-    # Make the month and day a pseudo real number.
-    my $m_d = "$month.$day";
-
     # Construct our lists of pseudo real number dates.
     my %inverted = _invert_holy_days ($year);
     my @sorted = sort { $a <=> $b } keys %inverted;
 
+    # Make the month and day a pseudo real number.
+    my $m_d = "$month.$day";
     my $holy_date;
 
     # Find the first date greater than the one provided.
@@ -329,12 +330,10 @@ sub next_holy_day {  # {{{
         }
     }
 
-    # If one was not found, just grab the last date in the list.
+    # If one was not found, grab the last date in the list.
     $holy_date = $sorted[-1] unless $holy_date;
 
-    return wantarray
-        ? ( $inverted{$holy_date} => [ split /\./, $holy_date ] )
-        : "$inverted{$holy_date}: $holy_date";
+    return $inverted{$holy_date};
 }  # }}}
 
 # Helper functions {{{
@@ -342,9 +341,12 @@ sub next_holy_day {  # {{{
 sub _setup_date_comparison {  # {{{
     my ($y, $m, $d, $s, $e) = @_;
 
+    # Dates are encoded as decimals.
     my ($start_month, $start_day) = split /\./, $s;
     my ($end_month, $end_day) = split /\./, $e;
 
+    # Slide either the start or end year, given the month we're
+    # looking at.
     my ($start_year, $end_year) = ($y, $y);
     if ($end_month < $start_month) {
         if ($m == $start_month) {
@@ -383,17 +385,11 @@ sub _build_date {  # {{{
     $date{month}++ unless $date{month} == -1;
 
     # Set the year.
-    # NOTE: This, as of yet, cryptic algorithm lifted from Arman
-    # Danesh's "bahaidate".
-    if (($month < MARCH) 
-        ||
+    # Algorithm lifted from Danesh's "bahaidate".
+    $date{year} = ($month < MARCH) ||
         ($month == MARCH && $day < YEAR_START_DAY)
-    ) {
-        $date{year} = $year - FIRST_YEAR;
-    }
-    else {
-        $date{year} = $year - (FIRST_YEAR - 1);
-    }
+        ? $year - FIRST_YEAR
+        : $year - (FIRST_YEAR - 1);
 
     $date{year_name} = (CYCLE_YEAR)[($date{year} - 1) % FACTOR];
     $date{cycle_year} = $date{year} % FACTOR;
@@ -409,11 +405,10 @@ sub _build_date {  # {{{
     # ($D_y,$D_m,$D_d, $Dh,$Dm,$Ds, $dst) = Timezone ();
     $date{timezone} = (Timezone ())[3];
 
-    # Get the holy day!
-    my $m_d = $month .'.'. sprintf ('%02d', $day);
+    # Get the holy day.
     my %inverted = _invert_holy_days ($year);
-    $date{holy_day} = { $inverted{$m_d} => HOLY_DAYS->{$inverted{$m_d}} }
-        if exists $inverted{$m_d};
+    my $m_d = sprintf '%d.%02d', $month, $day;
+    $date{holy_day} = $inverted{$m_d} if exists $inverted{$m_d};
 
     return wantarray ? %date : as_string (\%date, %args);
 }  # }}}
@@ -423,19 +418,21 @@ sub _invert_holy_days {  # {{{
 
     my %inverted;
 
-    while (my ($name, $dates) = each %{ HOLY_DAYS() }) {
-        # Pre-pad the day number with a zero.
-        $inverted{ sprintf '%d.%02d', @$dates[0,1] } = $name;
+    while (my ($name, $date) = each %{ HOLY_DAYS() }) {
+        $inverted{$date->[0]} = $name;
 
         # Does this date contain a day span?
-        if (@$dates == 3) {
+        if (@$date > 1) {
             # Increment the Ayyam-i-Ha day if we are in a leap year.
-            $dates->[2]++ if $name eq 'Ayyam-i-Ha' && leap_year ($year);
+            $date->[1]++ if $name eq 'Ayyam-i-Ha' && leap_year ($year);
+
+            for (1 .. $date->[1] - 1) {
+                (undef, my $month, my $day) = Add_Delta_Days(
+                    $year, split (/\./, $date->[0]), $_
+                );
 
-            for (1 .. $dates->[2] - 1) {
-                (undef, my $month, my $day) = Add_Delta_Days($year, @$dates[0,1], $_);
                 # Pre-pad the day number with a zero.
-                $inverted{ sprintf '%d.%02d', $month, $day} = $name;
+                $inverted{ sprintf '%d.%02d', $month, $day } = $name;
             }
         }
     }
@@ -449,7 +446,7 @@ __END__
 
 =head1 NAME
 
-Date::Baha::i - Compute the numeric and named Baha'i date.
+Date::Baha::i - Convert to and from Baha'i dates.
 
 =head1 SYNOPSIS
 
@@ -471,18 +468,18 @@ Date::Baha::i - Compute the numeric and named Baha'i date.
       day   => $day,
   );
 
-  ($year, $month, $day) = from_bahai (
-      year  => $bahai_year,
-      month => $bahai_month,
-      day   => $bahai_day,
-  );
   $date = from_bahai (
       year  => $bahai_year,
       month => $bahai_month,
       day   => $bahai_day,
   );
 
-  %holy_day = next_holy_day ($year, $month, $day);
+  ($year, $month, $day) = from_bahai (
+      year  => $bahai_year,
+      month => $bahai_month,
+      day   => $bahai_day,
+  );
+
   $holy_day = next_holy_day ($year, $month, $day);
 
   @cycles = cycles ();
@@ -494,24 +491,112 @@ Date::Baha::i - Compute the numeric and named Baha'i date.
 
 =head1 ABSTRACT
 
-This package outputs a (numeric and named) Baha'i date from a standard 
-epoch time stamp or year, month, day triple.
+This package converts between Baha'i and standard dates.
 
 =head1 DESCRIPTION
 
 This package renders the Baha'i date from two standard date formats -
-epoch time and the Gregorian year/month/day triple.  It is not a date
-arithmetic calculator or two-way converter.  It simply takes a
-standard date and converts it to the Baha'i representation.
+epoch time and a year/month/day triple.  It also converts a Baha'i 
+date to standard ymd format.
+
+This package is not a date arithmetic calculator.  It simply takes a 
+standard or Baha'i date and converts it to the reverse representation.
+
+The following passages are excerpts from the SEE ALSO section links:
+
+What we usually call the Baha'i calendar is technically called the 
+Badi calendar.  The word "Badi" means "Wonderful" and was the name 
+of several people of importance in Baha'i history, most notably the 
+youth who volunteered to carry a Tablet from Baha'u'llah to 
+Nasiri'd-Din Shah and was upon its delivery tortured and killed.  An 
+alternate translation of the word, used in the calendar itself, is 
+"Beginning".  But regardless of how the calendar came to be called 
+the Badi calendar, it was created by the Bab, and Baha'u'llah 
+specified a few of the details that His Forerunner had not provided.
+
+The number nineteen has a special significance for Baha'is.  It was 
+common in Persian mystical writings to utilize a system of numerical 
+values to convey meanings beyond what mere words could impart.
+Within this system, words are assigned numerical values, and 
+relationships between words can be implied based upon these values. 
+The word "vahid", meaning unity, has the numerical value of nineteen,
+and is often used by the Bab and Baha'u'llah when specifying the 
+quantity nineteen.  So the number nineteen, in addition to being a 
+quantity, also is evocative of the central teaching of the Baha'i 
+Faith, unity.  It forms the basis not only of the calendar, but also 
+was integral to the structure of the Persian Bayan (the Bab's Book of 
+laws); is found in Baha'u'llah's laws concerning dowries, the payment 
+of Huquq'u'llah, certain fines, and various prayers; and is even seen 
+in the history of the Faith, as Baha'u'llah's public declaration of 
+His mission took place nineteen years after the Bab's declaration.
+
+Now we come to the days of the month themselves. As is the case with 
+Jewish and Islamic reckoning, the day begins at sunset, rather than 
+at midnight.  For most of us, this takes a bit of getting used to! 
+It becomes important because certain things happen on specific days. 
+The first day of each Baha'i month is designated as a Feast day.  The 
+Feast is a community gathering that incorporates worship, community 
+business, and socializing.  It is the foundation of Baha'i community 
+life and is primarily administrative in nature.  When Baha'is gather 
+for the Feast of (say) Baha, the first month of the year, you might 
+think that the date on which they should gather is, in the Gregorian 
+calendar, March 21st.  But they may actually hold their Feasts 
+anytime between sunset on March 20th and before sunset on March 21st. 
+That time period is the first day of Baha.  Holy Days are also 
+reckoned in this fashion, as are the times for the start and end of 
+the Fast.
+
+Finally, for those who like to go into excruciating detail, the Bab 
+also spoke of time periods longer than a year.  He grouped years into 
+"Vahids" of nineteen years each, and gave each Vahid a name.  (It is 
+here that the word "Badi" appears, as the name of the sixteenth year 
+in the cycle.)  He further grouped the Vahids themselves into sets of 
+nineteen to create a time period called a "Kull-i-Shay" (literally, 
+"all things").  One Kull-i-Shay is therefore 361 years.
+
+Text taken from
+C<http://www.planetbahai.org/articles/2003/ar032103a.html>
+
+This calendar was instituted by the Baha'i spiritual leader 
+Baha'u'llah, who stated that it should begin in the Gregorian year 
+1844 at the (northern) Spring equinox, which is the traditional 
+Iranian New Year. According to calendars rules, the year begins at the
+sunset following the equinox, but up to now the practice in the West 
+has been to start the year at sunset on 20 March. This is usually 
+shown as 21 March, with the understanding that the day begins on the 
+evening before. In the Middle East, Baha'is start the year at the 
+sunset in Tehran following the equinox, and the Baha'i Universal House
+of Justice has not yet decided on the rules of the calendar to be used
+by all (Reingold and Dershowitz: Calendrical Calculations 2001). For 
+now, I present the calendar as used in the West.
+
+Baha'u'llah proclaimed the fulfillment of all religions and the unity 
+of humankind, and the calendar is designed to be a world calendar, 
+(relatively) free of cultural baggage. It is an entirely solar 
+calendar, without even the vestige of previously lunar months as in 
+the Gregorian Calendar.
+
+Text taken from
+C<http://www.moonwise.co.uk/year/159bahai.htm>
 
 The Baha'i year is based on the solar year of 365 days, five hours and
 some fifty minutes. Each year is divided into nineteen months of 
 nineteen days each with four Intercalary Days (five in a leap year), 
 called Ayyam-i-Ha which Baha'u'llah specified should precede the 
-nineteenth month. New Year's Day (Naw Ruz) falls on the Spring 
-Equinox. This usually occurs on 21 March but if the Equinox falls 
-after sunset on 21 March, Naw Ruz is to be celebrated on 22 March 
-because the Baha'i day begins at sunset.
+nineteenth month.
+
+The days of the Baha'i week are;
+
+  1. Jalal    - Glory (Saturday)
+  2. Jamal    - Beauty (Sunday)
+  3. Kaml     - Perfection (Monday)
+  4. Fidal    - Grace (Tuesday)
+  5. 'Idal    - Justice (Wednesday)
+  6. Istijlal - Majesty (Thursday)
+  7. Istiqlal - Independence (Friday)
+
+The Baha'i day of rest is Isiqlal (Friday) and the Baha'i day begins 
+and ends at sunset.
 
 The names of the months in the Baha'i (Badi) calendar were given by 
 the Bab, who drew them from the nineteen names of God invoked in a 
@@ -538,27 +623,7 @@ prayer said during the month of fasting in Shi'ih Islam. They are:
   * Ayyam-i-Ha  - Days of Ha (26 February - 1 March))
   19. 'Ala      - Loftiness (2 March - 20 March)
 
-The days of the Baha'i week are;
-
-  1. Jalal    - Glory (Saturday)
-  2. Jamal    - Beauty (Sunday)
-  3. Kaml     - Perfection (Monday)
-  4. Fidal    - Grace (Tuesday)
-  5. 'Idal    - Justice (Wednesday)
-  6. Istijlal - Majesty (Thursday)
-  7. Istiqlal - Independence (Friday)
-
-The Baha'i day of rest is Isiqlal (Friday) and the Baha'i day begins 
-and ends at sunset.
-
-Each of the days of the month is also given the name of one of the 
-attributes of God. The names are the same as those of the nineteen 
-months. Thus Naw-Ruz, the first day of the first month, would be 
-considered the 'day of Baha of the month Baha'. If it fell on a 
-Saturday, the first day of the Baha'i week, it would be the 'day of 
-jalal'.
-
-Ayyam-i-Ha
+Ayyam-i-Ha:
 
 Literally, Days of Ha (i.e. the letter Ha, which in the abjad system 
 has the numerical value of 5). Intercalary Days. The four days (five 
@@ -603,10 +668,10 @@ There are eleven Holy Days which Baha'is celebrate. On [many] of these
 days, all work should cease. They are listed in chronological order 
 according to the Baha'i calendar.
 
-* Naw Ruz - March 21
+* Naw Ruz - (Generally) March 21
 
 Literally, New Day. The Baha'i New Year. Like the ancient Persian New 
-Year, it occurs on the spring equinox, which generally falls on 21 
+Year, it occurs on the Spring equinox, which generally falls on 21 
 March. If the equinox falls after sunset on 21 March, Naw Ruz is 
 celebrated on 22 March, since the Baha'i day begins at sunset. For the
 present, however, the celebration of Naw Ruz is fixed on 21 March. In 
@@ -617,7 +682,7 @@ work is to be suspended.
 
 * Ridvan
 
-First day - 21 April; Ninth day - 29 April; Twelfth day - 2 May
+First day - 21 April; Ninth day - 29 April; Twelfth (last) day - 2 May
 
 The Ridvan (pronouced "riz-wan") festival commemorates the first 
 public declaration by Baha'u'llah of His Station and mission (in 
@@ -671,67 +736,12 @@ who are ill, travelers, the elderly, pregnant women and nursing
 mothers are exempt from the fast.
 
 Text taken from
-
-L<http://www.bahaindex.com/calendar.html>
-
-Baha'i Calendar
-
-(days start on the evening before the Gregorian date given)
-
-This new calendar was instituted by the Baha'i spiritual leader 
-Baha'u'llah, who stated that it should begin in the Gregorian year 
-1844 at the (northern) Spring equinox, which is the traditional 
-Iranian New Year. According to calendars rules, the year begins at the
-sunset following the equinox, but up to now the practice in the West 
-has been to start the year at sunset on 20 March. This is usually 
-shown as 21 March, with the understanding that the day begins on the 
-evening before. In the Middle East, Baha'is start the year at the 
-sunset in Tehran following the equinox, and the Baha'i Universal House
-of Justice has not yet decided on the rules of the calendar to be used
-by all (Reingold and Dershowitz: Calendrical Calculations 2001). For 
-now, I present the calendar as used in the West.
-
-Baha'u'llah proclaimed the fulfillment of all religions and the unity 
-of humankind, and the calendar is designed to be a world calendar, 
-(relatively) free of cultural baggage. It is an entirely solar 
-calendar, without even the vestige of previously lunar months as in 
-the Gregorian Calendar. It has nineteen months of nineteen days, with 
-some extra days before the last month. The cycle of 19 names is used 
-for the names of both the months and the days.
-
-The Vahid is a period of 19 years, and a Kull-i-Shay is 19 Vahids (361
-years). These cycles have another set of names: Alif (A,1), Ba (B,2), 
-Ab (Father,3), Dal (D,4), Bab (Gate,5), Vav (V,6), Abad (Eternity,7), 
-Jad (Generosity,8), Baha (Splendour,9), Hubb (Love,10), Bahhaj 
-(Delightful,11), Javab (Answer,12), Ahad (Single,13), Vahhab 
-(Bountiful,14), Vidad (Affection,15), Badi (Beginning,16), Bahi 
-(Luminous,17), Abha (Most Luminous,18), Vahid (Unity,19).
-
-The weekly seven day cycle is still used, and the day of rest is 
-Friday.
-
-Text taken from
-
-L<http://www.moonwise.co.uk/year/159bahai.htm>
+C<http://www.bahaindex.com/calendar.html>
 
 =head1 EXPORTED FUNCTIONS
 
 =head2 to_bahai
 
-  # Return a hash in array context.
-  %bahai_date = to_bahai ();
-  %bahai_date = to_bahai (
-      epoch => time,
-      use_gmtime => $use_gmtime,
-      %args,
-  );
-  %bahai_date = to_bahai (
-      year  => $year,
-      month => $month,
-      day   => $day,
-      %args,
-  );
-
   # Return a string in scalar context.
   $bahai_date = to_bahai ();
   $bahai_date = to_bahai (
@@ -746,6 +756,20 @@ L<http://www.moonwise.co.uk/year/159bahai.htm>
       %args,
   );
 
+  # Return a hash in array context.
+  %bahai_date = to_bahai ();
+  %bahai_date = to_bahai (
+      epoch => time,
+      use_gmtime => $use_gmtime,
+      %args,
+  );
+  %bahai_date = to_bahai (
+      year  => $year,
+      month => $month,
+      day   => $day,
+      %args,
+  );
+
 This function returns either a string or a hash of the Baha'i date 
 names and numbers from either epoch seconds or a year, month, day 
 triple.
@@ -772,15 +796,15 @@ hash with the following keys:
 
 =head2 from_bahai
 
-  # Return a ymd triple in array context.
-  ($year, $month, $day) = from_bahai (
+  # Return a y/m/d string in scalar context.
+  $date = from_bahai (
       year  => $bahai_year,
       month => $bahai_month,
       day   => $bahai_day,
   );
 
-  # Return a y/m/d string in scalar context.
-  $date = from_bahai (
+  # Return a ymd triple in array context.
+  ($year, $month, $day) = from_bahai (
       year  => $bahai_year,
       month => $bahai_month,
       day   => $bahai_day,
@@ -823,10 +847,10 @@ representation.
 Here are some handy examples (newlines added for readability):
 
   short numeric:
-  7, 1/1/159
+  1/1/159
 
   short numeric with TZ:
-  7, 1/1/159, -6
+  1/1/159, -6
 
   long numeric:
   7th day of the week, 1st day of the 1st month, year 159,
@@ -874,16 +898,10 @@ Here are some handy examples (newlines added for readability):
 
 =head2 next_holy_day
 
-  %holy_day = next_holy_day ($year, $month, $day);
   $holy_day = next_holy_day ($year, $month, $day);
 
-This function returns the first holy day after the provided date
-triple as either a hash (in array context) or a string (in scalar 
-context).
-
-In array context, a hash with a single key (the name of the holy day)
-and a two or three element array reference of [month, day, duration]
-as the value.
+This function returns the name of the first holy day after the 
+provided date triple.
 
 =head2 cycles
 
@@ -937,15 +955,10 @@ L<Lingua::Num2Word>
 
 =head1 TO DO
 
-Factor out the fugly _invert_holy_days function.
-
-Make the test suite test every possible date over two years, 
-including a leap year.
-
 Optionally output unicode.
 
 Base the date computation on the time of day (the Baha'i day begins at 
-Sunset) - and the location longitude/latitude.
+Sunset) and the location longitude/latitude.
 
 Overload localtime and gmtime, just to be cool?
 
@@ -953,29 +966,17 @@ Overload localtime and gmtime, just to be cool?
 
 Hi Kirsten  : )
 
-=head1 DISCLAIMER
+=head1 SEE ALSO
 
-I have no designs or intentions to be compatible with other date and 
-time representations (such as ICal or DateTime).  That is for those 
-external packages and the people who care to do the ultra-simple 
-conversions.
+C<http://www.projectpluto.com/calendar.htm#bahai> (Very interesting)
 
-This package is simple from the outside and requires only that which 
-is necessary to represent the Baha'i date.  That is, you give it a 
-date, and it returns the Baha'i representation.
+The following are partially quoted above:
 
-(Having said that, please see the TO DO section about wanting to do
-two-way conversions.)
+C<http://www.planetbahai.org/articles/2003/ar032103a.html>
 
-=head1 REFERENCES
+C<http://www.bahaindex.com/calendar.html>
 
-L<http://www.projectpluto.com/calendar.htm#bahai>
-
-L<http://www.bahaindex.com/calendar.html>
-
-L<http://www.moonwise.co.uk/year/159bahai.htm>
-
-L<http://www.perl.com/pub/a/2003/03/13/datetime.html>
+C<http://www.moonwise.co.uk/year/159bahai.htm>
 
 =head1 AUTHOR
 
